// Copyright (C) 2013 S&P Capital IQ
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.spcapitaliq.platformexecuting

import java.io.File
import java.util.zip.ZipEntry

import scalaz.{@@, Tag}
import scalaz.std.anyVal._
import scalaz.std.string._
import scalaz.syntax.equal._
import scalaz.syntax.semigroup._
import scalaz.syntax.std.option._

/** Marker for `ZipEntry` names. */
final class ZipEntName private()

object ZipEntName {
  private[platformexecuting]
  def apply(s: String): String @@ ZipEntName = Tag(s)

  /** Read the entry name from `z`. */
  def from(z: ZipEntry): String @@ ZipEntName = Tag(z.getName)

  private[this] val zebn = """([^/]+)/?$""".r

  private[platformexecuting]
  def zipEntBaseName(zen: String @@ ZipEntName): String @@ ZipEntName =
    ZipEntName(zebn findFirstMatchIn zen cata (_ group 1, ""))

  private[platformexecuting]
  def zipEntSplit(zen: String @@ ZipEntName
                 ): (String @@ ZipEntName, String @@ ZipEntName) = {
    val slash = zen.lastIndexOf('/'.toInt)
    Tag.subst[String, ({type λ[α] = (α, α)})#λ, ZipEntName
        ](if (slash < 0) ("", zen)
          else (zen substring (0, slash),
                zen substring (slash + 1)))
  }

  private[platformexecuting]
  def zipEntToNative(zen: String @@ ZipEntName): String =
    if (File.separatorChar === '/') zen
    else zen replace (File.separator, "/")

  private[platformexecuting]
  def relativizeZipEnt(root: String @@ ZipEntName, zen: String @@ ZipEntName
                      ): String @@ ZipEntName = {
    val pref = (root:String) |+| (if (root endsWith "/") "" else "/")
    val here = if (root endsWith "/") root.substring(0, root.length - 1) else root
    apply(if (zen startsWith pref) (zen substring pref.length)
          else if (zen === here) ""
          else zen)
  }
}
